package com.ycwl.basic.storage;

import com.ycwl.basic.storage.adapters.AliOssAdapter;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.storage.adapters.LocalStorageAdapter;
import com.ycwl.basic.storage.entity.StorageConfig;
import com.ycwl.basic.storage.enums.StorageType;
import com.ycwl.basic.storage.exceptions.StorageUndefinedException;
import com.ycwl.basic.storage.exceptions.StorageUnsupportedException;

import java.util.HashMap;
import java.util.Map;

public class StorageFactory {
    public static IStorageAdapter get(StorageType storageType, StorageConfig config) {
        IStorageAdapter adapter = get(storageType);
        adapter.setConfig(config);
        return adapter;
    }

    public static IStorageAdapter get(StorageType storageType) {
        switch (storageType) {
            case LOCAL:
                return new LocalStorageAdapter();
            case ALI_OSS:
                return new AliOssAdapter();
            default:
                throw new StorageUnsupportedException(storageType.getType());
        }
    }

    public static IStorageAdapter get(String type) {
        StorageType storageType = StorageType.getStorageType(type);
        if (storageType == null) {
            throw new StorageUnsupportedException(type);
        }
        return get(storageType);
    }

    public static Map<String, IStorageAdapter> definedName = new HashMap<>();

    public static void register(String name, IStorageAdapter adapter) {
        definedName.put(name, adapter);
    }

    public static IStorageAdapter use(String name) {
        IStorageAdapter adapter = definedName.get(name);
        if (adapter == null) {
            throw new StorageUndefinedException(name);
        }
        return adapter;
    }
}
